@@ -22,23 +22,24 @@ import asyncio
 import random
 import sys
 import logging
+from contextlib import contextmanager
 from typing import TypeVar, List, Optional, Generic, Union, cast, TYPE_CHECKING
 
 try:
     # helper module should work in no-window mode
     from PyQt5.QtCore import QModelIndex, QSize, Qt, pyqtSignal, QSortFilterProxyModel, \
         QAbstractListModel
-    from PyQt5.QtGui import QPalette, QFontMetrics, QColor
+    from PyQt5.QtGui import QPalette, QFontMetrics, QColor, QPainter
     from PyQt5.QtWidgets import QApplication, QScrollArea, QWidget
 except ImportError:
     pass
 
-from feeluown.utils import aio
+from feeluown.utils.aio import run_afn, run_fn
 from feeluown.utils.reader import AsyncReader, Reader
 from feeluown.utils.typing_ import Protocol
 from feeluown.excs import ProviderIOError, ResourceNotFound
-from feeluown.library import NotSupported, ModelType, BaseModel
-from feeluown.models.uri import reverse
+from feeluown.library import ModelNotFound, ModelType, BaseModel
+from feeluown.library import reverse
 
 
 if TYPE_CHECKING:
@@ -239,7 +240,9 @@ class ItemViewNoScrollMixin:
                 # qt will trigger fetchMore when the last row is
                 # inside the viewport, so we always hide the last
                 # two row to ensure fetch-more will not be
-                # triggered automatically
+                # triggered automatically.
+                #
+                # qt triggers fetchMore when user scrolls down to bottom.
                 index = self._last_visible_index()
                 rect = self.visualRect(index)
                 height = self.sizeHint().height() - int(rect.height() * 1.5) - \
@@ -424,17 +427,11 @@ class ReaderFetchMoreMixin(Generic[T]):
                     if count == step:
                         break
                 return items
-            future = aio.create_task(fetch())
-            future.add_done_callback(self._async_fetch_cb)
+            task = run_afn(fetch)
         else:
             assert isinstance(reader, Reader)
-            try:
-                items = reader.read_range(self.rowCount(), step + self.rowCount())
-            except ProviderIOError:
-                logger.exception('fetch more items failed')
-                self._fetch_more_cb(None)
-            else:
-                self._fetch_more_cb(items)
+            task = run_fn(reader.read_range, self.rowCount(), step + self.rowCount())
+        task.add_done_callback(self._async_fetch_cb)
 
     def on_items_fetched(self: ModelUsingReader[T], items: List[T]):
         begin = len(self._items)
@@ -451,10 +448,17 @@ class ReaderFetchMoreMixin(Generic[T]):
     def _async_fetch_cb(self: ModelUsingReader[T], future):
         try:
             items = future.result()
+        except ProviderIOError as e:
+            logger.error(f'async fetch more items failed, reason: {e}')
+            self._fetch_more_cb(None)
         except:  # noqa
             logger.exception('async fetch more items failed')
             self._fetch_more_cb(None)
         else:
+            if not items:
+                # The reader should not return empty list when fetching more items,
+                # maybe something wrong with the reader.
+                logger.warning('async fetch more items return empty list')
             self._fetch_more_cb(items)
 
 
@@ -489,8 +493,8 @@ def fetch_cover_wrapper(app: GuiApp):
         # If the song is a v1 model, just fallback to use its album cover.
         if not is_v2_model:
             try:
-                upgraded_song = await aio.run_fn(library.song_upgrade, model)
-            except NotSupported:
+                upgraded_song = await run_fn(library.song_upgrade, model)
+            except ModelNotFound:
                 cb(None)
             else:
                 await fetch_song_pic_from_album(upgraded_song.album, cb)
@@ -509,8 +513,8 @@ def fetch_cover_wrapper(app: GuiApp):
 
         # Image is not in cache.
         try:
-            upgraded_song = await aio.run_fn(library.song_upgrade, model)
-        except (NotSupported, ResourceNotFound):
+            upgraded_song = await run_fn(library.song_upgrade, model)
+        except ResourceNotFound:
             cb(None)
         else:
             # Try to fetch with pic_url first.
@@ -538,11 +542,7 @@ def fetch_cover_wrapper(app: GuiApp):
             cb(content)
             return
 
-        try:
-            img_url = await aio.run_fn(library.model_get_cover, model)
-        except NotSupported:
-            img_url = ''
-
+        img_url = await run_fn(library.model_get_cover, model)
         return await fetch_image_with_cb(img_uid, img_url, cb)
 
     async def fetch_model_cover(model, cb):
@@ -553,6 +553,27 @@ def fetch_cover_wrapper(app: GuiApp):
     return fetch_model_cover
 
 
+def random_solarized_color():
+    return QColor(random.choice(list(SOLARIZED_COLORS.values())))
+
+
+@contextmanager
+def painter_save(painter: QPainter):
+    painter.save()
+    yield
+    painter.restore()
+
+
+def secondary_text_color(palette: QPalette):
+    text_color: QColor = palette.color(QPalette.Text)
+    if text_color.lightness() > 150:
+        non_text_color = text_color.darker(140)
+    else:
+        non_text_color = text_color.lighter(150)
+    non_text_color.setAlpha(100)
+    return non_text_color
+
+
 # https://ethanschoonover.com/solarized/
 SOLARIZED_COLORS = {
     'yellow':    '#b58900',
