@@ -17,8 +17,7 @@ from PyQt5.QtWidgets import (
 
 from feeluown.utils import aio
 from feeluown.utils.dispatch import Signal
-from feeluown.library import ModelState, ModelFlags
-from feeluown.models import ModelExistence
+from feeluown.library import ModelState
 
 from feeluown.gui.mimedata import ModelMimeData
 from feeluown.gui.helpers import ItemViewNoScrollMixin, ReaderFetchMoreMixin
@@ -250,12 +249,8 @@ class BaseSongsTableModel(QAbstractTableModel):
         # If song's state is `not_exists` or `cant_upgrade`, the album and
         # artist columns are disabled.
         incomplete = False
-        if ModelFlags.v2 & song.meta.flags:
-            if song.state in (ModelState.not_exists, ModelState.cant_upgrade):
-                incomplete = True
-        else:
-            if song and song.exists == ModelExistence.no:
-                incomplete = True
+        if song.state in (ModelState.not_exists, ModelState.cant_upgrade):
+            incomplete = True
         if incomplete:
             if index.column() != Column.song:
                 flags = no_item_flags
@@ -647,12 +642,13 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
         menu.addAction(add_to_playlist_action)
 
         # remove song action
-        if self.remove_song_func is not None:
-            remove_song_action = QAction('移除歌曲', menu)
-            remove_song_action.triggered.connect(
-                lambda: self._remove_by_indexes(indexes))
-            menu.addSeparator()
-            menu.addAction(remove_song_action)
+        remove_song_action = QAction('移除歌曲', menu)
+        remove_song_action.triggered.connect(
+            lambda: self._remove_by_indexes(indexes))
+        menu.addSeparator()
+        menu.addAction(remove_song_action)
+        if self.remove_song_func is None:
+            remove_song_action.setDisabled(True)
 
         model = self.model()
         models = [model.data(index, Qt.UserRole) for index in indexes]
