@@ -63,12 +63,12 @@ try:
 except ImportError:
     # pydantic<2.0
     from pydantic import validator
-    identifier_validator = validator('identifier', pre=True)
+    identifier_validator = validator('identifier', pre=True)  # type: ignore
     pydantic_version = 1
 
-from feeluown.models import ModelType, ModelExistence, ModelStage, ModelFlags, AlbumType
-from feeluown.models import SearchType  # noqa
 from feeluown.utils.utils import elfhash
+from .base import ModelType, ModelFlags, AlbumType
+from .base import SearchType  # noqa
 from .model_state import ModelState
 
 
@@ -101,6 +101,15 @@ def fmt_artists(artists: List['BriefArtistModel']) -> str:
     return fmt_artists_names([artist.name for artist in artists])
 
 
+def get_duration_ms(duration):
+    if duration is not None:
+        seconds = duration / 1000
+        m, s = seconds / 60, seconds % 60
+    else:
+        m, s = 0, 0
+    return '{:02}:{:02}'.format(int(m), int(s))
+
+
 # When a model is fully supported (with v2 design), it means
 # the library has implemented all features(functions) for this model.
 # You can do anything with model v2 without model v1.
@@ -156,9 +165,6 @@ class BaseModel(_BaseModel):
     source: str = 'dummy'
     state: ModelState = ModelState.artificial
 
-    #: (DEPRECATED) for backward compact
-    exists: ModelExistence = ModelExistence.unknown
-
     @identifier_validator
     def int_to_str(cls, v):
         # Old version pydantic convert int to str implicitly.
@@ -217,41 +223,6 @@ class BaseBriefModel(BaseModel):
     """
     meta: Any = ModelMeta.create(is_brief=True)
 
-    @classmethod
-    def from_display_model(cls, model):
-        """Create a new model from an old model in display stage.
-
-        This method never triggers IO operations.
-        """
-        # Due to the display_property mechanism, it is unsafe to
-        # get attribute of other stage model property.
-        assert model.stage is ModelStage.display
-        data = {'state': cls._guess_state_from_exists(model.exists)}
-        for field in cls.__fields__:
-            if field in ('state', 'meta'):
-                continue
-            if field in ('identifier', 'source', 'exists'):
-                value = object.__getattribute__(model, field)
-            else:
-                if field in model.meta.fields_display:
-                    value = getattr(model, f'{field}_display')
-                else:
-                    # For example, BriefVideoModel has field `artists_name` and
-                    # the old model does not have such display field.
-                    value = ''
-            data[field] = value
-        return cls(**data)
-
-    @classmethod
-    def _guess_state_from_exists(cls, exists):
-        if exists == ModelExistence.no:
-            state_value = ModelState.not_exists
-        elif exists == ModelExistence.unknown:
-            state_value = ModelState.artificial
-        else:
-            state_value = ModelState.exists
-        return state_value
-
 
 class BaseNormalModel(BaseModel):
     meta: Any = ModelMeta.create(is_normal=False)
@@ -267,6 +238,9 @@ class BriefSongModel(BaseBriefModel):
     album_name: str = ''
     duration_ms: str = ''
 
+    def __str__(self):
+        return f'{self.source}:{self.title}•{self.artists_name}'
+
 
 class BriefVideoModel(BaseBriefModel):
     meta: Any = ModelMeta.create(ModelType.video, is_brief=True)
@@ -297,16 +271,21 @@ class BriefUserModel(BaseBriefModel):
     name: str = ''
 
 
-class SongModel(BaseNormalModel):
+class SongModel(BriefSongModel, BaseNormalModel):
     """
     ..versionadded: 3.8.11
         The `pic_url` field.
     """
     meta: Any = ModelMeta.create(ModelType.song, is_normal=True)
     title: str
-    album: Optional[BriefAlbumModel] = None
+    album: Optional[TAlbum] = None
     artists: List[BriefArtistModel]
     duration: int  # milliseconds
+    # A playlist can consist of multiple songs and a song can have many children.
+    # The differences between playlist's songs and song' children is that
+    # a child of a song usually have no album and artists. They share some
+    # metadata with the parent song.
+    children: List['TSong'] = []
 
     genre: str = ''
     date: str = ''  # For example: 2020-12-11 00:00:00, 2020-12-11T00:00:00Z
@@ -319,25 +298,17 @@ class SongModel(BaseNormalModel):
     # to fetch a image url of the song.
     pic_url: str = ''
 
-    @property
-    def artists_name(self):
-        return fmt_artists(self.artists)
+    def model_post_init(self, _):
+        super().model_post_init(_)
+        self.artists_name = fmt_artists(self.artists)
+        self.album_name = self.album.name if self.album else ''
+        self.duration_ms = get_duration_ms(self.duration)
 
-    @property
-    def album_name(self):
-        return self.album.name if self.album else ''
-
-    @property
-    def duration_ms(self):
-        if self.duration is not None:
-            seconds = self.duration / 1000
-            m, s = seconds / 60, seconds % 60
-        else:
-            m, s = 0, 0
-        return '{:02}:{:02}'.format(int(m), int(s))
+    def __str__(self):
+        return f'{self.source}:{self.title}•{self.artists_name}'
 
 
-class UserModel(BaseNormalModel):
+class UserModel(BriefUserModel, BaseNormalModel):
     meta: Any = ModelMeta.create(ModelType.user, is_normal=True)
     name: str = ''
     avatar_url: str = ''
@@ -363,7 +334,7 @@ class CommentModel(BaseNormalModel):
     root_comment_id: Optional[str] = None
 
 
-class ArtistModel(BaseNormalModel):
+class ArtistModel(BriefArtistModel, BaseNormalModel):
     meta: Any = ModelMeta.create(ModelType.artist, is_normal=True)
     name: str
     pic_url: str
@@ -372,7 +343,7 @@ class ArtistModel(BaseNormalModel):
     description: str
 
 
-class AlbumModel(BaseNormalModel):
+class AlbumModel(BriefAlbumModel, BaseNormalModel):
     """
     .. versionadded:: 3.8.12
         The `song_count` field.
@@ -398,9 +369,9 @@ class AlbumModel(BaseNormalModel):
     description: str
     released: str = ''  # format: 2000-12-27.
 
-    @property
-    def artists_name(self):
-        return fmt_artists(self.artists)
+    def model_post_init(self, _):
+        super().model_post_init(_)
+        self.artists_name = fmt_artists(self.artists)
 
 
 class LyricModel(BaseNormalModel):
@@ -409,28 +380,20 @@ class LyricModel(BaseNormalModel):
     trans_content: str = ''
 
 
-class VideoModel(BaseNormalModel):
+class VideoModel(BriefVideoModel, BaseNormalModel):
     meta: Any = ModelMeta.create(ModelType.video, is_normal=True)
     title: str
     artists: List[BriefArtistModel]
     duration: int
     cover: str
 
-    @property
-    def artists_name(self):
-        return fmt_artists(self.artists)
-
-    @property
-    def duration_ms(self):
-        if self.duration is not None:
-            seconds = self.duration / 1000
-            m, s = seconds / 60, seconds % 60
-        else:
-            m, s = 0, 0
-        return '{:02}:{:02}'.format(int(m), int(s))
+    def model_post_init(self, _):
+        super().model_post_init(_)
+        self.artists_name = fmt_artists(self.artists)
+        self.duration_ms = get_duration_ms(self.duration)
 
 
-class PlaylistModel(BaseBriefModel):
+class PlaylistModel(BriefPlaylistModel, BaseNormalModel):
     meta: Any = ModelMeta.create(ModelType.playlist, is_normal=True)
     # Since modelv1 playlist does not have creator field, it is set to optional.
     creator: Optional[BriefUserModel] = None
