@@ -3,19 +3,22 @@ import warnings
 import logging
 import random
 from enum import IntEnum, Enum
-from typing import Optional
+from typing import Optional, TYPE_CHECKING
 
-from feeluown.excs import ProviderIOError
+from feeluown.excs import ModelNotFound, ProviderIOError
 from feeluown.utils import aio
-from feeluown.utils.aio import run_fn
+from feeluown.utils.aio import run_fn, run_afn
 from feeluown.utils.dispatch import Signal
 from feeluown.utils.utils import DedupList
 from feeluown.player import Metadata, MetadataFields
 from feeluown.library import (
-    MediaNotFound, SongProtocol, ModelType, NotSupported, ResourceNotFound
+    MediaNotFound, SongModel, ModelType, ResourceNotFound,
 )
 from feeluown.media import Media
-from feeluown.models.uri import reverse
+from feeluown.library import reverse
+
+if TYPE_CHECKING:
+    from feeluown.app import App
 
 logger = logging.getLogger(__name__)
 
@@ -71,10 +74,10 @@ class PlaylistMode(IntEnum):
 
 
 class Playlist:
-    def __init__(self, app, songs=None, playback_mode=PlaybackMode.loop,
+    def __init__(self, app: 'App', songs=None, playback_mode=PlaybackMode.loop,
                  audio_select_policy='hq<>'):
         """
-        :param songs: list of :class:`feeluown.models.SongModel`
+        :param songs: list of :class:`feeluown.library.SongModel`
         :param playback_mode: :class:`feeluown.player.PlaybackMode`
         """
         self._app = app
@@ -126,6 +129,9 @@ class Playlist:
         #    The *songs_removed* and *songs_added* signal.
         self.songs_removed = Signal()  # (index, count)
         self.songs_added = Signal()  # (index, count)
+        # .. versionadded:: 3.9.0
+        #    The *play_model_handling* signal.
+        self.play_model_handling = Signal()
 
         self._app.player.media_finished.connect(self._on_media_finished)
 
@@ -397,6 +403,9 @@ class Playlist:
             previous_song = self._get_good_song(base=current_index - 1, direction=-1)
         return previous_song
 
+    async def a_next(self):
+        self.next()
+
     def next(self) -> Optional[asyncio.Task]:
         if self.next_song is None:
             self.eof_reached.emit()
@@ -416,7 +425,7 @@ class Playlist:
         return self.set_current_song(self.previous_song)
 
     @property
-    def current_song(self) -> Optional[SongProtocol]:
+    def current_song(self) -> Optional[SongModel]:
         """Current song
 
         return None if there is no current song
@@ -424,7 +433,7 @@ class Playlist:
         return self._current_song
 
     @current_song.setter
-    def current_song(self, song: Optional[SongProtocol]):
+    def current_song(self, song: Optional[SongModel]):
         self.set_current_song(song)
 
     def set_current_song(self, song) -> Optional[asyncio.Task]:
@@ -455,8 +464,6 @@ class Playlist:
 
         If the song is bad, then this will try to use a standby in Playlist.normal mode.
         """
-        song_str = f'{song.source}:{song.title_display} - {song.artists_name_display}'
-
         target_song = song  # The song to be set.
         media = None        # The corresponding media to be set.
 
@@ -464,61 +471,73 @@ class Playlist:
             media = await self._prepare_media(song)
         except MediaNotFound as e:
             if e.reason is MediaNotFound.Reason.check_children:
-                # TODO: maybe we can just add children to playlist?
-                self._app.show_msg(f'{song_str} 的播放资源在孩子节点上，将孩子节点添加到播放列表')
-                self.mark_as_bad(song)
-                logger.info(f'{song_str} has children, replace the current playlist')
-                song = await run_fn(self._app.library.song_upgrade, song)
-                if song.children:
-                    self.batch_add(song.children)
-                    await self.a_set_current_song(song.children[0])
-                else:
-                    self.next()
+                await self.a_set_current_song_children(song)
                 return
 
-            logger.info(f'{song_str} has no valid media, mark it as bad')
+            logger.info(f'no media found for {song} due to {e}, mark it as bad')
             self.mark_as_bad(song)
-
-            # if mode is fm mode, do not find standby song,
-            # just skip the song
-            if self.mode is PlaylistMode.fm:
-                self.next()
-                return
-
-            self._app.show_msg(f'{song_str} is invalid, try to find standby')
-            logger.info(f'try to find standby for {song_str}')
-            standby_candidates = await self._app.library.a_list_song_standby_v2(
-                song,
-                self.audio_select_policy
-            )
-            if standby_candidates:
-                standby, media = standby_candidates[0]
-                self._app.show_msg(f'Song standby was found in {standby.source} ✅')
-                # Insert the standby song after the song
-                if song in self._songs and standby not in self._songs:
-                    index = self._songs.index(song)
-                    self._songs.insert(index + 1, standby)
-                    self.songs_added.emit(index + 1, 1)
-                target_song = standby
-            else:
-                self._app.show_msg('Song standby not found')
         except ProviderIOError as e:
             # FIXME: This may cause infinite loop when the prepare media always fails
             logger.error(f'prepare media failed: {e}, try next song')
+            run_afn(self.a_next)
+            return
         except Exception as e:  # noqa
             # When the exception is unknown, we mark the song as bad.
             self._app.show_msg(f'prepare media failed due to unknown error: {e}')
             logger.exception('prepare media failed due to unknown error, '
                              'so we mark the song as a bad one')
             self.mark_as_bad(song)
-            self.next()
-            return
         else:
             assert media, "media must not be empty"
 
+        # The song has no media, try to find and use standby unless it is in fm mode.
+        if media is None:
+            # if mode is fm mode, do not find standby song, just skip the song.
+            if self.mode is PlaylistMode.fm:
+                run_afn(self.a_next)
+                return
+            target_song, media = await self.find_and_use_standby(song)
+
         metadata = await self._prepare_metadata_for_song(target_song)
         self.pure_set_current_song(target_song, media, metadata)
 
+    async def a_set_current_song_children(self, song):
+        # TODO: maybe we can just add children to playlist?
+        self._app.show_msg(f'{song} 的播放资源在孩子节点上，将孩子节点添加到播放列表')
+        self.mark_as_bad(song)
+        logger.info(f'{song} has children, replace the current playlist')
+        song = await run_fn(self._app.library.song_upgrade, song)
+        if song.children:
+            self.batch_add(song.children)
+            await self.a_set_current_song(song.children[0])
+        else:
+            run_afn(self.a_next)
+        return
+
+    async def find_and_use_standby(self, song):
+        self._app.show_msg(f'{song} is invalid, try to find standby')
+        logger.info(f'try to find standby for {song}')
+        standby_candidates = await self._app.library.a_list_song_standby_v2(
+            song,
+            self.audio_select_policy
+        )
+        if standby_candidates:
+            standby, media = standby_candidates[0]
+            msg = f'song standby was found in {standby.source} ✅'
+            logger.info(msg)
+            self._app.show_msg(msg)
+            # Insert the standby song after the song
+            if song in self._songs and standby not in self._songs:
+                index = self._songs.index(song)
+                self._songs.insert(index + 1, standby)
+                self.songs_added.emit(index + 1, 1)
+            return standby, media
+
+        msg = 'song standby not found'
+        logger.info(msg)
+        self._app.show_msg(msg)
+        return song, None
+
     def pure_set_current_song(self, song, media, metadata=None):
         if song is None:
             self._current_song = None
@@ -534,7 +553,7 @@ class Playlist:
 
         if song is not None:
             if media is None:
-                self.next()
+                run_afn(self.a_next)
             else:
                 # Note that the value of model v1 {}_display may be None.
                 kwargs = {}
@@ -556,7 +575,7 @@ class Playlist:
         })
         try:
             song = await aio.run_fn(self._app.library.song_upgrade, song)
-        except (NotSupported, ResourceNotFound):
+        except ResourceNotFound:
             return metadata
         except:  # noqa
             logger.exception(f"fetching song's meta failed, song:'{song.title_display}'")
@@ -567,7 +586,7 @@ class Playlist:
         if song.album is not None:
             try:
                 album = await aio.run_fn(self._app.library.album_upgrade, song.album)
-            except (NotSupported, ResourceNotFound):
+            except ResourceNotFound:
                 pass
             except:  # noqa
                 logger.warning(
@@ -599,7 +618,7 @@ class Playlist:
         })
         try:
             video = await aio.run_fn(self._app.library.video_upgrade, video)
-        except NotSupported as e:
+        except ModelNotFound as e:
             logger.warning(f"can't get cover of video due to {str(e)}")
         else:
             metadata[MetadataFields.artwork] = video.cover
@@ -607,6 +626,7 @@ class Playlist:
 
     async def _prepare_media(self, song):
         task_spec = self._app.task_mgr.get_or_create('prepare-media')
+        task_spec.disable_default_cb()
         if self.watch_mode is True:
             try:
                 mv_media = await task_spec.bind_blocking_io(
@@ -671,6 +691,9 @@ class Playlist:
 
         .. versionadded: 3.7.14
         """
+        # Stop the player so that user know the action is working.
+        self._app.player.stop()
+        self.play_model_handling.emit()
         task = self.set_current_model(model)
         if task is not None:
             def cb(future):
@@ -680,4 +703,5 @@ class Playlist:
                     logger.exception('play model failed')
                 else:
                     self._app.player.resume()
+                    logger.info(f'play a model ({model}) succeed')
             task.add_done_callback(cb)
